{% extends "blog/base.html" %}

{% block content %}

<div class="jumbotron">
<h1>dj-hitcount</h1>
<p>Welcome to the dj-hitcount example project.  Three different views are demonstrated:
<dl class="dl-horizontal">
    <dt>ajax</dt><dd> for a jQuery based implementation using the hitcount template tags</dd>
    <dt>detail</dt><dd> for a generic detail view that injects the hitcount into the context</dd>
    <dt>detail-with-count</dt><dd> for a second generic detail view that also process/counts the request as a hit or not</dd>
</dl></p>
<p>Please post any questions or issues on GitHub.  Enjoy!</p>
{% if post_list %}
<ul>
{% for posturl in post_views %}
<li>{{posturl}}
<ul>
{% for post in post_list %}
<li><a href="{% url posturl post.pk %}">{{ post.title }}</a></li>
{% endfor %}
</ul>
</li>
{% endfor %}
</ul>
{% else %}
    <p>No posts are available.</p>
{% endif %}
</div>

{%endblock%}
