# Generated by Django 3.2.4 on 2021-06-16 20:26
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('hitcount', '0004_auto_20200704_0933'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='BlacklistIP',
            new_name='BlockedIP',
        ),
        migrations.RenameModel(
            old_name='BlacklistUserAgent',
            new_name='BlockedUserAgent',
        ),
        migrations.AlterModelOptions(
            name='blockedip',
            options={'verbose_name': 'Blocked IP', 'verbose_name_plural': 'Blocked IPs'},
        ),
        migrations.AlterModelOptions(
            name='blockeduseragent',
            options={'verbose_name': 'Blocked User Agent', 'verbose_name_plural': 'Blocked User Agents'},
        ),
        migrations.AlterField(
            model_name='hit',
            name='ip',
            field=models.CharField(db_index=True, editable=False, max_length=40, null=True),
        ),
        migrations.AlterModelTable(
            name='blockedip',
            table=None,
        ),
        migrations.AlterModelTable(
            name='blockeduseragent',
            table=None,
        ),
    ]
