{% load static %}
<!DOCTYPE html>
<html lang="en">

<head>
  <meta charset="utf-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <title>dj-hitcount example project</title>
  <!-- Bootstrap -->
  <link rel="stylesheet" href="{% static 'vendor/bootstrap.min.css' %}">
  <link rel="stylesheet" href="{% static 'vendor/bootstrap-theme.min.css' %}">
  <!-- HTML5 Shim and Respond.js IE8 support of HTML5 elements and media queries -->
  <!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
  <!--[if lt IE 9]>
      <script src="https://oss.maxcdn.com/html5shiv/3.7.2/html5shiv.min.js"></script>
      <script src="https://oss.maxcdn.com/respond/1.4.2/respond.min.js"></script>
    <![endif]-->
  <style>
  body {
    padding-top: 70px;
    padding-bottom: 30px;
  }

  .theme-showcase .navbar .container {
    width: auto;
  }
  </style>
</head>

<body role="document">
  <!-- Fixed navbar -->
  <div class="navbar navbar-inverse navbar-fixed-top" role="navigation">
    <div class="container">
      <div class="navbar-header">
        <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse">
          <span class="sr-only">Toggle navigation</span>
          <span class="icon-bar"></span>
          <span class="icon-bar"></span>
          <span class="icon-bar"></span>
        </button>
        <a class="navbar-brand" href="/">dj-hitcount</a>
      </div>
      <div class="navbar-collapse collapse">
        <ul class="nav navbar-nav">

          {% if post_list %}
          {% for posturl in post_views %}
          <li class="dropdown">
            <a href="#" class="dropdown-toggle" data-toggle="dropdown">
            {{posturl}} <span class="caret"></span>
            </a>
            <ul class="dropdown-menu" role="menu">
              {% for post in post_list %}
              <li><a href="{% url posturl post.pk %}">{{ post.title }}</a></li>
              {% endfor %}
            </ul>
          </li>
          {% endfor %} {% endif %}
          <li><a href="/admin/">Admin</a></li>
          <li><a href="https://github.com/abhiabhi94/dj-hitcount">Github</a></li>
          <li><a href="https://dj-hitcount.rtfd.org">Docs</a></li>
        </ul>
      </div>
      <!--/.nav-collapse -->
    </div>
  </div>
  <div class="container theme-showcase" role="main">
    {% block content %}{% endblock %}
  </div>
  <!-- /container -->
  <!-- jQuery (necessary for Bootstrap's JavaScript plugins) -->
  <script src="{% static 'vendor/jquery-2.1.4.min.js' %}"></script>
  <script src="{% static 'vendor/bootstrap.min.js' %}"></script>
  {# This template block is here so we can insert the JavaScript on correct template #}
  {% block inline_javascript %}{% endblock %}
</body>

</html>
