HITCOUNT_KEEP_HIT_ACTIVE = {"days": 7}

HITCOUNT_HITS_PER_IP_LIMIT = 0

HITCOUNT_HITS_PER_SESSION_LIMIT = 0

HITCOUNT_EXCLUDE_USER_GROUP = ()

HITCOUNT_KEEP_HIT_IN_DATABASE = {"days": 30}

HITCOUNT_USE_IP = True

HITCOUNT_HITCOUNT_MODEL = "hitcount.HitCount"
